/* To launch this program from within Mathematica use:
 *   In[1]:= link = Install["matlabio"]
 *
 * Or, launch this program from a shell and establish a
 * peer-to-peer connection.  When given the prompt Listen on:
 * type a port name. ( On Unix platforms, a port name is a
 * number less than 65536.  On a Mac it's an arbitrary word.)
 * Then, from within Mathematica use:
 *   In[1]:= link = Install["portname", LinkMode->Connect]
 */

/* To build this file on Fedora Core 5:

1) /usr/local/Wolfram/Mathematica/5.2/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions/mprep -o matlabio.tm.c matlabio.tm

2) Start Matlab, and run:

DEBUGGING VERSION:
mex -g -f /usr/local/matlab/matlab_r2006a/bin/matopts.sh -I/usr/local/Wolfram/Mathematica/5.2/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions -L/usr/local/Wolfram/Mathematica/5.2/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions -lML matlabio.tm.c matlabio.c -o matlabio -v

RELEASE VERSION:
mex -O -f /usr/local/matlab/matlab_r2006a/bin/matopts.sh -I/usr/local/Wolfram/Mathematica/5.2/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions -L/usr/local/Wolfram/Mathematica/5.2/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions -lML matlabio.tm.c matlabio.c -o matlabio -v

To debug:

1) LD_LIBRARY_PATH=/usr/local/matlab/matlab_r2006a/bin/glnx86:/usr/local/matlab/matlab_r2006a/sys/os/glnx86 ddd ./matlabio

2) Set breakpoints

3) "run -linkcreate"

4) matlabio = Install[LinkConnect["40742@192.168.100.1,50399@192.168.100.1"]] with the appropriate output from linkcreate in step 3.

5) Now call the function from Mathematica and ddd will stop at the breakpoint.

*/

/* $Id: matlabio.c,v 1.1 2006/08/18 17:05:46 rodgers Exp $ */

#include <stdlib.h>
#include <string.h>
#include <mathlink.h>
#include "mat.h"

/* used to send messages via MathLink */
#define ERRBUFSIZE	200
#define MAXMESSAGEPARTS	4
static char *msgv[MAXMESSAGEPARTS];

/************** error handling ****************/
static
int send_message_to_mathematica(char *symb, char *tag, int msgc, char *msgv[])
{
	int pkt,i;
	
	MLPutFunction( stdlink, "EvaluatePacket", 1);
	MLPutFunction( stdlink, "Message", msgc+1);
	MLPutFunction( stdlink, "MessageName", 2);
	MLPutSymbol( stdlink, symb);
	MLPutString( stdlink, tag);
	for (i = 0; i < msgc; i++) MLPutString( stdlink, msgv[i]);
	MLEndPacket( stdlink);
	
	while( (pkt = MLNextPacket( stdlink)) && pkt != RETURNPKT)
	  MLNewPacket( stdlink);
	MLNewPacket( stdlink);
	
	return MLError( stdlink) == MLEOK;
}

void matlaberror(char *error_text)
/* Standard error handler */
/* This handler uses MathLink to */
/* return its error message. It also returns $Failed. */
{
	char errBuf[ERRBUFSIZE];
	sprintf(errBuf, "%s",error_text);
	msgv[0] = errBuf;
	send_message_to_mathematica("MATLABIO", "matlaberr", 1, msgv);
}

void MATLABMatrixNames(char *filename)
{
	MATFile *fp;
	char    **dir;
	int     ndir, i;

	fp = matOpen(filename, "r");
	if (fp == NULL) {
		matlaberror("Can't open mat file for reading");
		goto error;
	}
	dir = matGetDir(fp, &ndir);
	matClose(fp);

	if (dir == NULL) {
		matlaberror("Can't read directory");
		goto error;
	} else {
		MLPutFunction(stdlink, "List", ndir);
		for (i=0; i < ndir; i++) {
			MLPutString(stdlink, dir[i]);
		}
	}
	mxFree(dir);
	return;
error:
	MLPutSymbol(stdlink, "$Failed");
	return;
}

void MATLABReadMatrix(char *filename, char *matrixname)
{
	MATFile *fp;
	mxArray  *mp;
    double    *data, *currPtr, *realPtr, *imagPtr;
    long    dims[3];
    char    *heads[3];
    long    depth;
    int     row, col;

	/* I'd prefer to use aggregate initializers, but K&R compilers */
	/* such as that shipped standard by Sun are retarded in this */
	/* respect. */
	dims[0] = dims[1] = dims[2] = 0;
	heads[0] = "List"; heads[1] = "List"; heads[2] = "Complex";
	
	fp = matOpen(filename, "r");
	if (fp == NULL) {
		matlaberror("Can't open mat file for reading");
		goto error1;
	}
	mp = matGetVariable(fp, matrixname);
	matClose(fp);

	if (mp == NULL) {
		matlaberror("Can't find the requested matrix");
		goto error1;
	} else if (mxIsSparse(mp)) {
		matlaberror("The requested matrix is sparse");
		goto error2;
	} else if (mxIsChar(mp)) {
		matlaberror("The requested matrix is a string");
		goto error2;
	} else {
		dims[0] = mxGetM(mp);
		dims[1] = mxGetN(mp);
		dims[2] = 2;
		
		if (mxGetPi(mp) == NULL) { /* real matrix */
			depth = 2;
			realPtr = mxGetPr(mp);
			data = (double *)malloc(dims[0] * dims[1] * sizeof(double));
			if (!data) {
				matlaberror("Memory allocation failed in MATLABReadMatrix");
				goto error2;
			}
			for (currPtr = data, row = 0; row < dims[0]; row++)
				for (col = 0; col < dims[1]; col++)
					*(currPtr++) = realPtr[col*dims[0]+row];
		} else {    /* complex matrix */
			depth = 3;
			data = (double *)malloc(dims[0] * dims[1] * dims[2] * sizeof(double));
			if (!data) {
				matlaberror("Memory allocation failed in MATLABReadMatrix");
				goto error2;
			}
			realPtr = mxGetPr(mp);
			imagPtr = mxGetPi(mp);
			for (currPtr = data, row = 0; row < dims[0]; row++)
				for (col = 0; col < dims[1]; col++) {
					*(currPtr++) = realPtr[col*dims[0]+row];
					*(currPtr++) = imagPtr[col*dims[0]+row];
				}
		}
		MLPutDoubleArray(stdlink, data, dims, heads, depth);
		free(data);
		mxDestroyArray(mp);
	}
	return;
error2:
	mxDestroyArray(mp);
error1:
	MLPutSymbol(stdlink, "$Failed");
	return;
}

void MATLABWriteMatrix(char *filename, char *matrixname)
{
	MATFile *fp;
	mxArray  *mp;
	int		mlres;
    double    *data, *currPtr, *realPtr, *imagPtr;
    long    *dims;
    char    **heads;
    long    depth;
    int     row, col;

	fp = matOpen(filename, "u");
	if (fp == NULL) {
   	        MLNewPacket(stdlink);
		matlaberror("Can't open mat file for update");
		goto error1;
	}

	mlres = MLGetDoubleArray(stdlink, &data, &dims, &heads, &depth);
	if (!mlres) {
		matlaberror("MLGetDoubleArray failed in MATLABWriteMatrix");
		goto error2;
	} else {
		switch (depth) {
			case 1: /* 1D real */
				mp = mxCreateDoubleMatrix(1, dims[0], mxREAL);
				if (mp == NULL) {
					matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
					goto error3;
				} else {
					realPtr = mxGetPr(mp);
					for (currPtr = data, col = 0; col < dims[0]; col++)
						realPtr[col] = *(currPtr++);
				}
				break;
			case 2: /* 1D complex or 2D real */
				if (!strcmp(heads[1], "Complex")) {
					/* 1D complex */
					mp = mxCreateDoubleMatrix(1, dims[0], mxCOMPLEX);
					if (mp == NULL) {
						matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
						goto error3;
					} else {
						realPtr = mxGetPr(mp);
						imagPtr = mxGetPi(mp);
						for (currPtr = data, col = 0; col < dims[0]; col++) {
							realPtr[col] = *(currPtr++);
							imagPtr[col] = *(currPtr++);
						}
					}
				} else {
					/* 2D real */
					mp = mxCreateDoubleMatrix(dims[0], dims[1], mxREAL);
					if (mp == NULL) {
						matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
						goto error3;
					} else {
						realPtr = mxGetPr(mp);
						for (currPtr = data, row = 0; row < dims[0]; row++)
							for (col = 0; col < dims[1]; col++)
								realPtr[col*dims[0]+row] = *(currPtr++);
					}
				}
				break;
			case 3: /* 2D Complex */
				if (!strcmp(heads[2], "Complex")) {
					mp = mxCreateDoubleMatrix(dims[0], dims[1], mxCOMPLEX);
					if (mp == NULL) {
						matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
						goto error3;
					} else {
						realPtr = mxGetPr(mp);
						imagPtr = mxGetPi(mp);
						for (currPtr = data, row = 0; row < dims[0]; row++)
							for (col = 0; col < dims[1]; col++) {
								realPtr[col*dims[0]+row] = *(currPtr++);
								imagPtr[col*dims[0]+row] = *(currPtr++);
							}
					}
				} else {
					matlaberror("MATLAB can handle only 1D or 2D arrays");
					goto error3;
				}
				break;
			default:
				matlaberror("MATLAB can handle only 1D or 2D arrays");
				goto error3;
		}
		MLDisownDoubleArray(stdlink, data, dims, heads, depth);
	}
	/* mxSetName(mp, matrixname); */
	matPutVariable(fp, matrixname, mp);
	mxDestroyArray(mp);
	matClose(fp);
	MLPutString(stdlink, filename);
	return;
error3:
	MLDisownDoubleArray(stdlink, data, dims, heads, depth);
error2:
	matClose(fp);
error1:
	MLPutSymbol(stdlink, "$Failed");
	return;
}

void MATLABCreateMATFile(char *filename)
{
	MATFile *fp;

	fp = matOpen(filename, "w");
	if (fp == NULL) {
		matlaberror("Can't create mat file");
		goto error1;
	}
	matClose(fp);

	MLPutString(stdlink, filename);
	return;
error1:
	MLPutSymbol(stdlink, "$Failed");
	return;
}

int main(int argc, char *argv[])
{
    return MLMain(argc, argv);
}
